% Correction application Chapitre 6
% Macroconomie quantitative (2014, d. Ellipses) 
% par Christophe Cahn

% Ce fichier permet de reproduire les lements contenu
% dans le fichier App_6_RBC.pdf disponible sur le site
% www.christophecahn.fr/macroquant

% Filename : App_6_RBC.m
% Application : Matlab/Octave
% Date : 24-04-2014
% Christophe Cahn (c) 2014

addpath('../MMMtbx');

clear all; clc; close all;
 
% 6

% On commence par dclarer les paramtres en leur affectant leur valeur
BETAA 	= 0.984 ;
DELTAA 	= 0.025 ;
ALPHAA 	= 0.33 ;
THETAA	= 3.48 ;
RHOO 	= 0.98 ;
SIGMAA 	= 0.0072 ;

% On dfinit galement la taille du problme
m = 7;		% # de variables endognes
k = 1;		% # de variables exognes

% Calcul de l'tat stationnaire
RBAR = (1/BETAA - 1 + DELTAA) ;
Y_K  = RBAR/ALPHAA ;
K_H  = Y_K^(1/(ALPHAA-1)) ;
C_H  = K_H^ALPHAA - DELTAA*K_H ;
HBAR = 1/(1+THETAA/(1-ALPHAA)*(1-DELTAA*ALPHAA/RBAR));
KBAR = HBAR*K_H;
CBAR = HBAR*C_H;
IBAR = DELTAA*KBAR ;
YBAR = KBAR*Y_K ;
WBAR = (1-ALPHAA)*YBAR/HBAR ;

% Dfinition de l'quilibre log-linaris sous sa forme matricielle
% |E_t[Fx_{t+1} + Gx_t + Hx_{t-1} + Lz_{t+1} + Mz_t]=0|
% |z_{t+1} = Nz_t + eps_{t+1}|
% avec |x_t = [c_t, h_t, k_{t+1}, y_t, i_t, r_t, w_t]'|
phi_i = DELTAA*ALPHAA/(1/BETAA - 1 + DELTAA);
phi_c = 1 - phi_i;
phi_h = 1/(1-(1-ALPHAA)/(1+THETAA*phi_c));

F= [0 , 0 , 0 , 0 , 0 , 0 , 0 ;
	1 , 0 , 0 , BETAA*(1-DELTAA) - 1 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0 ;	
	0 , 0 , 0 , 0 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0 ];

G= [1     , phi_h   , 0                 ,-1                 , 0      , 0 , 0;
    -1    , 0       , 1-BETAA*(1-DELTAA), 0                 , 0      , 0 , 0;
    phi_c , 0 		, 0                 , -1                , phi_i  , 0 , 0;
    0     , 1-ALPHAA, 0                 , -1                , 0      , 0 , 0;
    0     , 0       , 1                 , 0                 ,-DELTAA , 0 , 0;
    0     , 0       , 0                 , 1/BETAA+DELTAA - 1, 0      ,-1 , 0;
    0     , -1      , 0                 , 1                 , 0      , 0 ,-1];                    

H= [0 , 0 , 0 , 0 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0 ;	
	0 , 0 , ALPHAA , 0 , 0 , 0 , 0 ;
	0 , 0 , -(1-DELTAA) , 0 , 0 , 0 , 0 ;
	0 , 0 , -(1/BETAA +DELTAA -1) , 0 , 0 , 0 , 0 ;
	0 , 0 , 0 , 0 , 0 , 0 , 0];

L = [0, 0, 0, 0, 0, 0, 0]';
M = [0, 0, 0, 1, 0, 0, 0]';
N = RHOO;

%%
% Dfinition de la structure |model|

%%
% Pour mmoire, l'ordre des variables est (c, h, k, y, i, r, w)'

model.F 		= F ;
model.G 		= G ;
model.H 		= H ;
model.L 		= L ;
model.M 		= M ;
model.N 		= N ;
model.SIGMAA 	= SIGMAA ;
model.nEndoVar 	= m ;
model.nExoVar  	= k ;

model = setfield(model, 'EndoNames', {	'consommation','heures travaillees','capital', 'production','investissement','taux interet','salaire'});		
model = setfield(model, 'ExoNames', {'choc technologique'});
model = setfield(model, 'etatstat', [CBAR, HBAR, KBAR, YBAR, IBAR, RBAR, WBAR]');
model = setfield(model, 'isLogLinear', 1);

%% 
% Rsolution du modle  anticipations rationnelles
% On suppose x_t = Px_{t-1} + Qz_t

[model.P, model.Q] = solveLinearDynamicModel(model);

%%
% Calcul et reprsentation des IRF

IrfLinearModel = computeIRF(model, 100, 1, [], '_logLinearModel');
print(1,'./App_6_fig_irf1.pdf','-dpdf')
print(2,'./App_6_fig_irf2.pdf','-dpdf')

% 7

% Initialisation
% randn("seed",13);
T       = 115;
nsim    = 50 ;
RES     = zeros(7,6,nsim);
z       = zeros( k, T );
x_hat   = zeros( m, T );
x_hp    = zeros( m, T );
corr_xy = zeros(m,1);

% itrations sur 50 simulations
for jj = 1:nsim
    % tirage d'une chronique de choc
    e_t = SIGMAA*randn( k, T );
    % initialisation du 1er point
    z(:,1) = e_t(:,1);
    x_hat(:,1) = model.Q*z(:,1);
    % calcul de la rcurrence
    for ii=2:T
        z(:,ii) = N*z(:,ii-1) + e_t(:,ii); 
        x_hat(:,ii) = model.P*x_hat(:,ii-1) + model.Q*z(ii);	
    end
    % calcul des niveaux
    x = (1+x_hat).*model.etatstat(:,ones(1,T));
    x_moy = mean(x')';
    % calcul de la composante cyclique pour chaque variable
    for ii=1:m
		[crap,tmp] = hptrend(log(x(ii,:))',1600);
		x_hp(ii,:) = tmp';
    end
    x_var = var(x_hp')';	% variance
    x_ect = 100*sqrt(x_var);	% cart-type
    x_ret = x_ect/x_ect(4);	% cart-type relatif // production
    for kk=1:m
		try
	        corr_xy(kk) = corr(x_hp(kk,:),x_hp(4,:));
		catch ME
			tmp = corrcoef(x_hp(kk,:),x_hp(4,:));
			corr_xy(kk) = tmp(1,2);
	    end
    end
    RES(:,:,jj) = [model.etatstat x_moy x_var x_ect x_ret corr_xy];
end;
RES = mean(RES,3); % moyenne sur les simulations
% calcul des moments

fprintf('Moments simules\n\n');
fprintf('   E-s   moy.  var.  ect. ret. corr\n');
fprintf('y %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(4,:));
fprintf('c %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(1,:));
fprintf('h %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(2,:));
fprintf('k %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(3,:));
fprintf('i %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(5,:));
fprintf('r %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(6,:));
fprintf('w %1.2f %5.2f %1.1e %1.2f %1.2f %1.2f\n', RES(7,:));

%%
% *8)* Effectuez  nouveau les questions \textbf{6}-\textbf{7} avec $\rho=0.2$ et
% comparez les rsultats.}



%%


